unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, SunTime, StdCtrls, ExtCtrls, Spin, ComCtrls; 

type
  TMainForm = class(TForm)
    SunTime: TSunTime;
    TimeZoneGroup: TGroupBox;
    UseSysTimeZone: TCheckBox;
    Latitude: TGroupBox;
    DateBox: TGroupBox;
    TargetDate: TDateTimePicker;
    GroupBox: TGroupBox;
    MorningLabel: TLabel;
    Sunrise: TLabel;
    Noon: TLabel;
    NoonLabel: TLabel;
    Sunset: TLabel;
    EveningLabel: TLabel;
    LatDegLabel: TLabel;
    LatDeg: TSpinEdit;
    LatMinLabel: TLabel;
    LatMin: TSpinEdit;
    LatSecLabel: TLabel;
    LatSec: TSpinEdit;
    Longitude: TGroupBox;
    LonDegLabel: TLabel;
    LogMinLabel: TLabel;
    LogSecLabel: TLabel;
    LonDeg: TSpinEdit;
    LonMin: TSpinEdit;
    LonSec: TSpinEdit;
    TimeZoneHours: TSpinEdit;
    TimeZoneHoursLabel: TLabel;
    TimeZoneMin: TSpinEdit;
    TimeZoneMinLabel: TLabel;
    LatDirLabel: TLabel;
    LatDir: TComboBox;
    LonDir: TComboBox;
    LonDirLabel: TLabel;
    Desc1Label: TLabel;
    Link1Label: TLabel;
    Zenith: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    ZenithDeg: TSpinEdit;
    ZenithMin: TSpinEdit;
    ZenithSec: TSpinEdit;
    ZenithDestanceMemo: TMemo;
    Desc2Label: TLabel;
    Link2Label: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure LatDegChange(Sender: TObject);
    procedure LatMinChange(Sender: TObject);
    procedure LatSecChange(Sender: TObject);
    procedure LatDirChange(Sender: TObject);
    procedure LonDegChange(Sender: TObject);
    procedure LonMinChange(Sender: TObject);
    procedure LonSecChange(Sender: TObject);
    procedure LonDirChange(Sender: TObject);
    procedure UseSysTimeZoneClick(Sender: TObject);
    procedure TimeZoneHoursChange(Sender: TObject);
    procedure TimeZoneMinChange(Sender: TObject);
    procedure TargetDateChange(Sender: TObject);
    procedure LinkLabelClick(Sender: TObject);
    procedure ZenithDegChange(Sender: TObject);
    procedure ZenithMinChange(Sender: TObject);
    procedure ZenithSecChange(Sender: TObject);
  private
    procedure UpdateTimeZone;
    procedure UpdateTimes;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  ShellAPI;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  UseSysTimeZone.Checked := SunTime.UseSysTimeZone;
  LatDeg.Value := SunTime.Latitude.Degrees;
  LatMin.Value := SunTime.Latitude.Minutes;
  LatSec.Value := SunTime.Latitude.Seconds;
  LatDir.ItemIndex := Ord(SunTime.Latitude.Dir);
  LonDeg.Value := SunTime.Longitude.Degrees;
  LonMin.Value := SunTime.Longitude.Minutes;
  LonSec.Value := SunTime.Longitude.Seconds;
  LonDir.ItemIndex := Ord(SunTime.Longitude.Dir);
  ZenithDeg.Value := SunTime.ZenithDistance.Degrees;
  ZenithMin.Value := SunTime.ZenithDistance.Minutes;
  ZenithSec.Value := SunTime.ZenithDistance.Seconds;
  TargetDate.Date := SunTime.Date;
  UpdateTimeZone;
  UpdateTimes;
end;

procedure TMainForm.LatDegChange(Sender: TObject);
begin
  SunTime.Latitude.Degrees := LatDeg.Value;
  UpdateTimes;
end;

procedure TMainForm.LatMinChange(Sender: TObject);
begin
  SunTime.Latitude.Minutes := LatMin.Value;
  UpdateTimes;
end;

procedure TMainForm.LatSecChange(Sender: TObject);
begin
  SunTime.Latitude.Seconds := LatSec.Value;
  UpdateTimes;
end;

procedure TMainForm.LatDirChange(Sender: TObject);
begin
  SunTime.Latitude.Dir := TLatitudeDir(LatDir.ItemIndex);
  UpdateTimes;
end;

procedure TMainForm.LonDegChange(Sender: TObject);
begin
  SunTime.Longitude.Degrees := LonDeg.Value;
  UpdateTimes;
end;

procedure TMainForm.LonMinChange(Sender: TObject);
begin
  SunTime.Longitude.Minutes := LonMin.Value;
  UpdateTimes;
end;

procedure TMainForm.LonSecChange(Sender: TObject);
begin
  SunTime.Longitude.Seconds := LonSec.Value;
  UpdateTimes;
end;

procedure TMainForm.LonDirChange(Sender: TObject);
begin
  SunTime.Longitude.Dir := TLongitudeDir(LonDir.ItemIndex);
  UpdateTimes;
end;

procedure TMainForm.UseSysTimeZoneClick(Sender: TObject);
begin
  SunTime.UseSysTimeZone := UseSysTimeZone.Checked;
  TimeZoneHours.Enabled := not SunTime.UseSysTimeZone;
  TimeZoneHoursLabel.Enabled := not SunTime.UseSysTimeZone;
  TimeZoneMin.Enabled := not SunTime.UseSysTimeZone;
  TimeZoneMinLabel.Enabled := not SunTime.UseSysTimeZone;
  if SunTime.UseSysTimeZone then UpdateTimeZone;
  UpdateTimes;
end;

procedure TMainForm.TimeZoneHoursChange(Sender: TObject);
begin
  SunTime.TimeZone := TimeZoneHours.Value + TimeZoneMin.Value / 60;
  UpdateTimes;
end;

procedure TMainForm.TimeZoneMinChange(Sender: TObject);
begin
  SunTime.TimeZone := TimeZoneHours.Value + TimeZoneMin.Value / 60;
  UpdateTimes;
end;

procedure TMainForm.TargetDateChange(Sender: TObject);
begin
  SunTime.Date := TargetDate.Date;
  if SunTime.UseSysTimeZone then UpdateTimeZone;
  UpdateTimes;
end;

procedure TMainForm.UpdateTimeZone;
var
  TZ: Extended;
begin
  TZ := SunTime.TimeZone;
  TimeZoneHours.OnChange := nil;
  try
    TimeZoneHours.Value := Trunc(TZ);
  finally
    TimeZoneHours.OnChange := TimeZoneHoursChange;
  end;
  TimeZoneMin.OnChange := nil;
  try
    TimeZoneMin.Value := Trunc(Abs(Frac(TZ)) * 60);
  finally
    TimeZoneMin.OnChange := TimeZoneHoursChange;
  end;
end;

procedure TMainForm.UpdateTimes;
begin
  if SunTime.Sunrise = 0 then
    Sunrise.Caption := 'N/A'
  else
    Sunrise.Caption := FormatDateTime('hh:nn:ss  dddddd', SunTime.Sunrise);
  if SunTime.Noon = 0 then
    Noon.Caption := 'N/A'
  else
    Noon.Caption := FormatDateTime('hh:nn:ss  dddddd', SunTime.Noon);
  if SunTime.Sunset = 0 then
    Sunset.Caption := 'N/A'
  else
    Sunset.Caption := FormatDateTime('hh:nn:ss  dddddd', SunTime.Sunset);
end;

procedure TMainForm.LinkLabelClick(Sender: TObject);
begin
  ShellExecute(0, 'open', PChar(TLabel(Sender).Hint), nil, nil, SW_NORMAL);
end;

procedure TMainForm.ZenithDegChange(Sender: TObject);
begin
  SunTime.ZenithDistance.Degrees := ZenithDeg.Value;
  UpdateTimes;
end;

procedure TMainForm.ZenithMinChange(Sender: TObject);
begin
  SunTime.ZenithDistance.Minutes := ZenithMin.Value;
  UpdateTimes;
end;

procedure TMainForm.ZenithSecChange(Sender: TObject);
begin
  SunTime.ZenithDistance.Seconds := ZenithSec.Value;
  UpdateTimes;
end;

end.
